//////////////////////////////////////////////////////////////////////////
////                               DS1302.C                           ////
////                     Driver for Real Time Clock                   ////
////                                                                  ////
////  rtc_init()                                   Call after power up////
////                                                                  ////
////  rtc_set_datetime(day,mth,year,dow,hour,min)  Set the date/time  ////
////                                                                  ////
////  rtc_get_date(day,mth,year,dow)               Get the date       ////
////                                                                  ////
////  rtc_get_time(hr,min,sec)                     Get the time       ////
////                                                                  ////
////  rtc_write_nvr(address,data)                  Write to NVR       ////
////                                                                  ////
////  data = rtc_read_nvr(address)                 Read from NVR      ////
////                                                                  ////
////  get_bcd(data)                              Convert data to BCD  ////
////                                                                  ////
////  rm_bcd(data)                               Convert data to int  ////
////                                                                  ////
//////////////////////////////////////////////////////////////////////////
////        (C) Copyright 1996,2003 Custom Computer Services          ////
//// This source code may only be used by licensed users of the CCS C ////
//// compiler.  This source code may only be distributed to other     ////
//// licensed users of the CCS C compiler.  No other use, reproduction////
//// or distribution is permitted without written permission.         ////
//// Derivative programs created using this software in object code   ////
//// form are not restricted in any way.                              ////
//////////////////////////////////////////////////////////////////////////

/*
	  Modificato per le proprie esigenze
*/


#ifndef RTC_SCLK

#define RTC_SCLK PIN_C3
#define RTC_IO   PIN_C4  //-> PULL UP !
#define RTC_RST  PIN_C5

#endif
#define RTC_R_HR 0x85
#define RTC_R_MIN 0x83
#define RTC_R_SEC 0x81
#define RTC_R_DAY 0x87
#define RTC_R_MTH 0x89
#define RTC_R_YY 0x8D

#define RTC_W_HR 0x84
#define RTC_W_MIN 0x82
#define RTC_W_SEC 0x80
#define RTC_W_DAY 0x86
#define RTC_W_MTH  0x88
#define RTC_W_YY 0x8c
 

void write_ds1302_byte(BYTE cmd);
void write_ds1302(BYTE cmd, BYTE data);
BYTE read_ds1302(BYTE cmd);
void rtc_init();
int get_bcd(BYTE data);
int rm_bcd(BYTE data);
void rtc_set_hour(int ss,int mm,int hh);
void rtc_set_date(int day,int mth,int year);
void rtc_read_date();
void rtc_read_hour();



/*
Funzione: void write_ds1302_byte(BYTE cmd)
Scopo: Scrive un comando al rtc
Ritorna: -
Parametri: 
	BYTE cmd: comando
Revisione: 1
Note: -
*/ 
void write_ds1302_byte(BYTE cmd) {
  
   INT I;
   for(i=0;i<=7;++i) {
      output_bit(RTC_IO, shift_right(&cmd,1,0) );
      output_high(RTC_SCLK);
      output_low(RTC_SCLK);
   }

}

/*
Funzione: void write_ds1302(BYTE cmd, BYTE data)
Scopo: Scrive un comando e un dato al rtc
Ritorna: -
Parametri: 
	BYTE cmd: comando
	BYTE data: dato da inviare
Revisione: 1
Note: -
*/ 
void write_ds1302(BYTE cmd, BYTE data) {

   output_high(RTC_RST);
   write_ds1302_byte(cmd);
   write_ds1302_byte(data);
   output_low(RTC_RST);
}

/*
Funzione: BYTE read_ds1302(BYTE cmd)
Scopo: Legge un registro dal rtc
Ritorna: 
	BYTE valore del registro
Parametri: 
	BYTE cmd: comando da inviare
Revisione: 1
Note: -
*/ 
BYTE read_ds1302(BYTE cmd) {
   BYTE i,data;

   output_high(RTC_RST);
   write_ds1302_byte(cmd);

   for(i=0;i<=7;++i) {
      shift_right(&data,1,input(RTC_IO));
      output_high(RTC_SCLK);
      delay_us(2);
      output_low(RTC_SCLK);
      delay_us(2);
   }
   output_low(RTC_RST);

   return(data);
}

/*
Funzione: void rtc_init()
Scopo: Init rtc
Ritorna: -
Parametri: -
Revisione: 1
Note: -
*/ 
void rtc_init() {
   BYTE x;
   output_low(RTC_RST);
   delay_us(2);
   output_low(RTC_SCLK);
   write_ds1302(0x8e,0);
   write_ds1302(0x90,0xa4);
   x=read_ds1302(0x81);
   if((x & 0x80)!=0)
     write_ds1302(0x80,0);
}

/*
Funzione: int get_bcd(BYTE data)
Scopo: Codifica in formato bcd
Ritorna:
	int valore codificato in bcd
Parametri: 
	BYTE data: dato da codificare
Revisione: 1
Note: -
*/ 
int get_bcd(BYTE data){
   int nibh;
   int nibl;

   nibh=data/10;
   nibl=data-(nibh*10);

   return((nibh<<4)|nibl);
}

/*
Funzione: int rm_bcd(BYTE data)
Scopo: Decodifica da formato bcd
Ritorna:
	int valore in bcd
Parametri: 
	BYTE data: dato decodificato
Revisione: 1
Note: -
*/ 
int rm_bcd(BYTE data){
   int i;
   i=data;
   data=(i>>4)*10;
   data=data+(i<<4>>4);

   return data;
}

/*
Funzione: void rtc_set_hour(int ss,int mm,int hh
Scopo: Imposta l'ora nel rtc
Ritorna:
Parametri: 
	int ss: secondi
	int mm: minuti
	int hh: ora 0..23
Revisione: 1.1
Note: -
*/ 
void rtc_set_hour(int ss,int mm,int hh){
   write_ds1302(RTC_W_SEC,get_bcd(ss));// importante, rispettare questo ordine !! prima i secondi !!
   write_ds1302(RTC_W_MIN,get_bcd(mm));
   write_ds1302(RTC_W_HR,get_bcd(hh));    
}

/*
Funzione: void rtc_set_date(int day,int mth,int year)
Scopo: Imposta l'ora nel rtc
Ritorna:
Parametri: 
	int day: giorno
	int mth: mese
	int year: anno
Revisione: 1
Note: -
*/ 
void rtc_set_date(int day,int mth,int year) {
  write_ds1302(RTC_W_DAY,get_bcd(day));
  write_ds1302(RTC_W_MTH,get_bcd(mth));
  write_ds1302(RTC_W_YY,get_bcd(year)); 
}

/*
Funzione: void rtc_read_date()
Scopo: Legge la data
Ritorna: -
Parametri: -
Revisione: 1
Note: -
*/ 
void rtc_read_date(){
   rtc.day = rm_bcd(read_ds1302(RTC_R_DAY));
   rtc.mth = rm_bcd(read_ds1302(RTC_R_MTH));
   rtc.yy = rm_bcd(read_ds1302(RTC_R_YY));
}

/*
Funzione: void rtc_read_date()
Scopo: Legge l'ora
Ritorna: -
Parametri: -
Revisione: 1
Note: -
*/ 
void rtc_read_hour(){
   rtc.hh = rm_bcd(read_ds1302(RTC_R_HR));
   rtc.mm = rm_bcd(read_ds1302(RTC_R_MIN));
   rtc.ss = rm_bcd(read_ds1302(RTC_R_SEC));
}
