/*****************************

	sensore.c

Rilasciato con licenza GNU GPL
	
Si ringrazia Acmesystems 
  per il codice da cui  stato 
  tratto spunto.	
	
******************************/

// Example of user space C program to read a humidity and temperature
// sensor Sensirion SHT71 (http://www.acmesystems.it/?id=89)

// Version improved by by Douglas Gilbert 
// Modificato da Manzoni Giovanni

//14 BIT TEMP, 12 BIT UMIDITY

// -----------------------------------------------------------------------
// SHT71 sensor define
// -----------------------------------------------------------------------

// Macro to set the data line direction
#define DATA_LINE_IN      input(SENS_DATA)
//#define DATA_LINE_OUT     gpiosetdir(PORTG,DIROUT,DATA_MSK)

#define DATA_LINE_LOW     output_low(SENS_DATA); delay_us(100);
#define DATA_LINE_HIGH    output_high(SENS_DATA); delay_us(100);
#define DATA_LINE_READ    input(SENS_DATA)

#define CLOCK_LINE_LOW    output_low(SENS_clk); delay_us(100);
#define CLOCK_LINE_HIGH   output_high(SENS_clk); delay_us(100);

#define CMD_READ_TEMP     	0x03 //011
#define CMD_READ_HUM      	0x05 //101
#define CMD_SOFT_RESET      0x1E //101
#define CMD_WRITE_STATUS    0x06 //110


static int SendByte(unsigned char byte1);
static void SendReset (void);
static int SendByte(unsigned char byte1);
static unsigned char ReadByte(int withack);
static unsigned long ReadTemperature(void);
static unsigned long ReadHumidity(void);

/*
Funzione: static void SendStart(void)
Scopo: Manda start sul bus
Ritorna: -
Parametri: -
Revisione: 1
Note: -
*/ 
static void SendStart(void){
	DATA_LINE_HIGH;
    CLOCK_LINE_HIGH;
    DATA_LINE_LOW;
	CLOCK_LINE_LOW;
    CLOCK_LINE_HIGH;
    DATA_LINE_HIGH;
    CLOCK_LINE_LOW;
    DATA_LINE_IN;
}

/*
Funzione: static void SendReset (void)
Scopo: Manda il reset al sensore
Ritorna: -
Parametri: -
Revisione: 1
Note: -
*/ 
static void SendReset (void){
    /*int k;
    DATA_LINE_HIGH; //HES
    for (k = 0; k < 12; k++) {
        CLOCK_LINE_HIGH;
        CLOCK_LINE_LOW;
    }
    */
    
    // SOFT RESET - HES
    SendStart();
    SendByte(CMD_SOFT_RESET);
    delay_ms(15);//> 11
}

/*
Funzione: static int SendByte(unsigned char byte1)
Scopo: Invia un byte al sensore
Ritorna: -
Parametri: -
Revisione: 1
Note: -
*/ 
static int SendByte(unsigned char byte1){
    unsigned char tempbyte;
    int k;

    //DATA_LINE_OUT;
    tempbyte = byte1;
    for (k = 0x80; k > 0; k /= 2) {

        if (tempbyte & k){
            DATA_LINE_HIGH;}
        else{
            DATA_LINE_LOW;}

        CLOCK_LINE_HIGH;
        CLOCK_LINE_LOW;
    }
    DATA_LINE_IN;
    CLOCK_LINE_HIGH;
    CLOCK_LINE_LOW;
    return 1;
}

/*
Funzione: static int SendByte(unsigned char byte1)
Scopo: Legge un byte dal sensore
Ritorna: 
	static unsigned char: byte letto
Parametri:
	withack: 1 = send the ACK; 0 = don't send the ACK
Revisione: 1
Note: -
*/ 
static unsigned char ReadByte(int withack){
    unsigned char tempbyte;
    int k;

    tempbyte = 0;
    for (k = 0x80; k > 0; k /= 2) {
        CLOCK_LINE_HIGH;
        if (DATA_LINE_READ) tempbyte |= k;
        delay_us(100);   //read at half Ton of clock    
        CLOCK_LINE_LOW;
    }
    if (withack) {
        // Acknowledge del byte
        DATA_LINE_LOW;
        CLOCK_LINE_HIGH;  //9th clock
        CLOCK_LINE_LOW;
        DATA_LINE_IN;
    } else {
        // Senza acknowledge
        DATA_LINE_HIGH;
        CLOCK_LINE_HIGH;
        CLOCK_LINE_LOW;
        DATA_LINE_IN;
    }
    return tempbyte;
}

/*
Funzione: static unsigned long ReadTemperature(void)
Scopo: Legge la temperatura
Ritorna: 
	static unsigned long: temperatura
Parametri: -
Revisione: 1
Note: -
*/ 
static unsigned long ReadTemperature(void){
    unsigned int Lsb,  Chk;
    unsigned long Msb,result;
    
    SendStart();
    if (! SendByte(CMD_READ_TEMP))
        return 0;
    while (DATA_LINE_READ); // while 1 wait procesisng on sensor
    delay_us(100);
    //ok data is available
    
    Msb = ReadByte(1);
    Lsb = ReadByte(1);
    Chk = ReadByte(0);
    
    Msb=Msb << 8;
    result= (long)(Msb) + (long)Lsb;
    return result; 
}

/*
Funzione: static unsigned long ReadHumidity(void)
Scopo: Legge l' umidit
Ritorna: 
	static unsigned long: temperatura
Parametri: -
Revisione: 1
Note: -
*/ 
static unsigned long ReadHumidity(void){
    unsigned int Lsb,  Chk;
    unsigned long Msb,result;

    SendStart();

    if (! SendByte(CMD_READ_HUM))
        return 0;
    while (DATA_LINE_READ) //11/55/210ms +-15%  ->  8/12/14 bit
        ;
    Msb = ReadByte(1);
    Lsb = ReadByte(1);
    Chk = ReadByte(0);
    Msb=Msb << 8;
    result= (long)(Msb) + (long)Lsb;
    return result;
}



